@@ -1,3 +1,8 @@
+"""
+This module implements classes that represents Certification Authorities (CA).
+"""
+import shutil
+
 import re
 
 import json
@@ -21,6 +26,12 @@ from SCAutolib.models.file import OpensslCnf
 
 class BaseCA:
     dump_file: Path = None
+    _ca_cert: Path = None
+    _ca_key: Path = None
+
+    @property
+    def cert(self):
+        return self._ca_cert
 
     def request_cert(self, csr, username: str, cert_out: Path):
         """
@@ -34,6 +45,7 @@ class BaseCA:
             Default None
         :type cert_out: pathlib.Path
         :return: None or path where the certificate is stored.
+        :rtype: None or pathlib.Path
         """
         ...
 
@@ -68,14 +80,14 @@ class BaseCA:
         Load CA from JSON file.
         :return: CA object
         """
-        with json_file.dump_file.open("r") as f:
+        with json_file.open("r") as f:
             cnt = json.load(f)
 
         if "_ipa_server_ip" in cnt.keys():
             ca = IPAServerCA(ip_addr=cnt["_ipa_server_ip"],
                              server_hostname=cnt["_ipa_server_hostname"],
-                             root_passwd=cnt["_ipa_server_root_password"],
-                             admin_passwd=cnt["_ipa_server_admin_password"],
+                             root_passwd=cnt["_ipa_server_root_passwd"],
+                             admin_passwd=cnt["_ipa_server_admin_passwd"],
                              client_hostname=cnt["_ipa_client_hostname"],
                              domain=cnt["_ipa_server_domain"],
                              realm=cnt["_ipa_server_realm"])
@@ -111,8 +123,8 @@ class LocalCA(BaseCA):
             conf_type="CA",
             filepath=self.root_dir.joinpath("ca.cnf"),
             replace=str(self.root_dir))
-        self._ca_cert: Path = self.root_dir.joinpath("rootCA.pem")
-        self._ca_key: Path = self.root_dir.joinpath("rootCA.key")
+        self._ca_cert = self.root_dir.joinpath("rootCA.pem")
+        self._ca_key = self.root_dir.joinpath("rootCA.key")
 
         self._serial: Path = self.root_dir.joinpath("serial")
         self._index: Path = self.root_dir.joinpath("index.txt")
@@ -127,8 +139,7 @@ class LocalCA(BaseCA):
             raise SCAutolibException("CNF file does not exist")
         self._ca_cnf = cnf
 
-    @property
-    def __dict__(self):
+    def to_dict(self):
         """
         Customising default property for better serialisation for storing to
         JSON format.
@@ -142,6 +153,31 @@ class LocalCA(BaseCA):
             dict_["_ca_cnf"] = str(self._ca_cnf.path)
         return dict_
 
+    @property
+    def is_installed(self):
+        """
+        Check if the local CA is installed
+        """
+        try:
+            result = all([self.root_dir.exists(),
+                          self._ca_cert.exists(),
+                          self._ca_key.exists(),
+                          self._ca_cnf.path.exists(),
+                          self._conf_dir.exists(),
+                          self._newcerts.exists(),
+                          self._certs.exists(),
+                          self._crl.exists(),
+                          self._serial.exists(),
+                          self._index.exists()])
+            if result and self._ca_pki_db.exists():
+                with self._ca_pki_db.open("r") as f:
+                    with self._ca_cert.open("r") as cert:
+                        result &= cert.read() in f.read()
+        except Exception as e:
+            logger.error(e)
+            return False
+        return result
+
     def setup(self):
         """
         Creates directory and file structure needed by local CA. If directory
@@ -251,7 +287,11 @@ class LocalCA(BaseCA):
         Remove the root directory of the CA
         """
         logger.warning(f"Removing local CA {self.root_dir}")
-        rmtree(self.root_dir, ignore_errors=True)
+        for file in self.root_dir.iterdir():
+            if file.is_file():
+                file.unlink()
+            elif file.is_dir():
+                shutil.rmtree(file)
         logger.info(f"Local CA from {self.root_dir} is removed")
 
 
@@ -268,6 +308,7 @@ class IPAServerCA(BaseCA):
     ``IPAServerCA.create()`` method.
     """
 
+    _ca_cert: Path = Path("/etc/ipa/ca.crt")
     _ipa_server_ip: str = None
     _ipa_server_hostname: str = None
     _ipa_server_domain: str = None
@@ -333,8 +374,7 @@ class IPAServerCA(BaseCA):
     def domain(self):
         return self._ipa_server_domain
 
-    @property
-    def __dict__(self):
+    def to_dict(self):
         """
         Customising default property for better serialisation for storing to
         JSON format.
