@@ -3,23 +3,27 @@ This module contains classes that represent configuration files. Each class
 contains information and methods to manipulate specific config file except
 the parent (File) class that is supposed to operate on general config file.
 
-basic operations on config files defined in this module:
-    create: create content of config file usually based on template.
-            Note, that some child classes may also update content of config
-            file if it already existed and backup original file.
-    set:    modify values of config files, add keys or sections
-            if necessary
-    save:   save modified content to config file
-    clean:  remove config file; note that some child classes may also restore
-            original config file if backup exists.
+Basic operations on config files defined in this module:
+    * create
+        creates content of config file usually based on template.
+        Note, that some child classes may also update content of config
+        file if it already existed and backup original file.
+    * set
+        modify values of config files, add keys or sections if necessary
+    * save
+        save modified content to config file
+    * clean
+        remove config file; note that some child classes may also restore
+        original config file if backup exists.
 """
 import os
 from configparser import ConfigParser
 from pathlib import Path
 from shutil import copy2
+from traceback import format_exc
 from typing import Union
 
-from SCAutolib import logger, TEMPLATES_DIR, LIB_BACKUP
+from SCAutolib import logger, TEMPLATES_DIR, LIB_BACKUP, run
 from SCAutolib.exceptions import SCAutolibException
 
 
@@ -27,14 +31,17 @@ class File:
     """
     This class defines an interface for generic operations on config files
 
-    create: create content of config file based on template file
-    set:    modify content of config files, add keys or sections if necessary
-    save:   save config file
-    clean:  remove config file
+    * create: create content of config file based on template file
+    * set:    modify content of config files, add keys or sections if necessary
+    * save:   save config file
+    * clean:  remove config file
 
-    Note:   Set method operates on 1) files compatible with ConfigParser (i.e.
-            files containing sections); 2) simple config files without sections.
-            Other formats of config files are not supported.
+    .. note:: Set method operates **only** on:
+
+     * files compatible with ConfigParser (i.e. files containing sections)
+     * simple config files without sections.
+
+     Other formats of config files are not supported.
     """
     _conf_file = None
     _template = None
@@ -79,9 +86,12 @@ class File:
         """
         Modify value in config file. Modification is made through the
         ConfigParser object if it is defined. If not, then key value pair
-        would be written to the file through normal `write()` method with
-        composed string in the following form `<key><separator><value>` (spaces
-        around key has to be specified as a part of the `separator` parameter).
+        would be written to the file through normal :code:`write()` method with
+        composed string in the following form :code:`<key><separator><value>`
+
+        .. note::
+            spaces around key has to be specified as a part of the
+            :code:`separator` parameter.
 
         :param key: value for this key will be updated
         :type key: str
@@ -145,8 +155,9 @@ class File:
         by line splitting the line on separator. First match wins and is
         returned.
 
-        If section is provided and the file can be parsed by the ConfigParser,
-        then this object would be used to look for the key.
+        If section is provided and the file can be parsed by the
+        :code:`ConfigParser`, then this object would be used to look for the
+        key.
 
         :param key: required key
         :param section: section where the key should be found
@@ -206,7 +217,7 @@ class File:
     def backup(self, name: str = None):
         """
         Save original file to the backup directory with given name. If name is
-        None, default name is <filename>.<extension>.backup
+        None, default name is :code:`<filename>.<extension>.backup`
 
         :param name: custom file name to be set for the file
         :type name: str
@@ -236,26 +247,19 @@ class SSSDConf(File):
     /etc/sssd/sssd.conf file.
 
     It is implemented as singleton, which allows to use class object
-    _default_parser as representation of content of sssd.conf file during
-    runtime.
+    :code:`_default_parser` as representation of content of sssd.conf file
+    during runtime.
 
     Intended use is to create/update and save config file in first runtime
-    and load content of config file to internal parser object in in following
+    and load content of config file to internal parser object in following
     runtimes.
-
-    Following methods are implemented:
-
-    create: create a content of internal parser object representing sssd.conf
-    set:  set or modify content of internal parser object representing sssd.conf
-    save: save content of internal parser object representing sssd.conf to
-          /etc/sssd/sssd.conf file
-    clean:  restore original state of /etc/sssd/sssd.conf file
     """
     __instance = None
     _template = Path(TEMPLATES_DIR, "sssd.conf")
     _conf_file = Path("/etc/sssd/sssd.conf")
     _backup_original = None
     _backup_default = LIB_BACKUP.joinpath('default-sssd.conf')
+    _changed = False
 
     def __new__(cls):
         if cls.__instance is None:
@@ -281,6 +285,25 @@ class SSSDConf(File):
         self._changes = ConfigParser()
         self._changes.optionxform = str
 
+    def __call__(self, key: str, value: Union[int, str, bool],
+                 section: str = None):
+        self.set(key, value, section)
+        self.save()
+        run("systemctl restart sssd", sleep=5)
+        return self
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if self._changed:
+            copy2(self._backup_default, self._conf_file)
+            self._changed = False
+        if exc_type is not None:
+            logger.error("Exception in virtual smart card context")
+            logger.error(format_exc())
+        run("systemctl restart sssd", sleep=5)
+
     def create(self):
         """
         Populate internal parser object with content from existing config file
@@ -324,8 +347,14 @@ class SSSDConf(File):
             self._changes.add_section(section)
 
         previous = self._changes.get(section, key, fallback="Not set")
+        if previous == value:
+            logger.info(f"A key '{key}' in section '{section}' is already set "
+                        f"to {value}. No changes in SSSD are required.")
+            return
 
         self._changes.set(section, key, value)
+        self._changed = True
+
         logger.info(f"Value is changed in section {self._changes[section]}")
         logger.debug(f"Old value in section [{section}] {key}={previous}")
         logger.debug(f"New value in section [{section}] {key}={value}")
@@ -333,6 +362,8 @@ class SSSDConf(File):
     def save(self):
         """
         Save content of config file stored in parser object to config file.
+
+        .. note: SSSD service restart is caller's responsibility.
         """
         with self._conf_file.open("w") as config:
             if len(self._changes.sections()) == 0:
@@ -351,6 +382,8 @@ class SSSDConf(File):
         """
         Removes sssd.conf file in case it was created by this package or
         restore original sssd.conf in case the file was modified.
+
+        .. note: SSSD service restart is caller's responsibility.
         """
 
         # FIXME: this implementation would not work in real usage. When setup
@@ -360,9 +393,12 @@ class SSSDConf(File):
         #  required attributes in JSON format load() method that would be used
         #  in other then setup runtimes to restore (load from JSON) all
         #  attributes of this object
-        self.clean()
+
         if self._backup_original:
             copy2(self._backup_original, self._conf_file)
+        else:
+            self.clean()
+        self._changed = False
 
     def update_default_content(self):
         """
@@ -392,11 +428,7 @@ class SSSDConf(File):
 class SoftHSM2Conf(File):
     """
     This class provide information and methods to create and modify
-    softhsm2.conf file. Following methods are implemented:
-
-    create: create content of internal file object representing softhsm2.conf
-    save:   save content of internal file object representing softhsm2.conf to
-            softhsm2.conf file
+    softhsm2.conf file.
     """
     _template = Path(TEMPLATES_DIR, "softhsm2.conf")
     _conf_file = None
@@ -429,7 +461,7 @@ class SoftHSM2Conf(File):
 
     def set(self, *args):
         """
-        Raise NotImplementedError as set method for softHSM2 is not implemented
+        :raise NotImplementedError: if this method is called on SoftHSM2Conf.
         """
         # parent class set method does not work as softHSM2 conf does not have
         # sections. Method do modify softHSM2 conf is not implemented
@@ -448,11 +480,7 @@ class SoftHSM2Conf(File):
 class OpensslCnf(File):
     """
     This class provides information and methods to create and modify
-    openssl cnf files. Following methods are implemented:
-
-    create: create content of internal file object representing openssl cnf file
-    save:  save content of internal file object representing openssl cnf file to
-           .cnf file specified by user
+    openssl cnf files.
     """
     _template = None
     _conf_file = None
