@@ -4,15 +4,16 @@ across the library. These functions are made based on library demands and are
 not attended to cover some general use-cases or specific corner cases.
 """
 import json
-
 from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.asymmetric import rsa
 from enum import Enum
 from pathlib import Path
 
-from SCAutolib import run, logger, TEMPLATES_DIR, LIB_DUMP_USERS
+from SCAutolib import run, logger, TEMPLATES_DIR, LIB_DUMP_USERS, LIB_DUMP_CAS
 from SCAutolib.exceptions import SCAutolibException
+from SCAutolib.models.CA import LocalCA, BaseCA, IPAServerCA
 from SCAutolib.models.card import Card
+from SCAutolib.models.file import OpensslCnf
 from SCAutolib.models.user import BaseUser
 
 
@@ -111,6 +112,7 @@ def _check_packages(packages):
     """
     missing = []
     for pkg in packages:
+        # Return code 1 means the package is not installed
         out = run(["rpm", "-q", pkg], return_code=[0, 1])
         if out.returncode == 1:
             logger.warning(f"Package {pkg} is required for the testing, "
@@ -126,11 +128,11 @@ def dump_to_json(obj):
     Store serialised object to the JSON file.
     """
     with obj.dump_file.open("w") as f:
-        json.dump(obj.__dict__, f)
+        json.dump(obj.to_dict(), f)
     logger.debug(f"Object {type(obj)} is stored to the {obj.dump_file} file")
 
 
-def user_factory(username):
+def user_factory(username, **kwargs):
     """
     Load user with given username from JSON file. If user have the card file
     linked, then load it as well.
@@ -146,7 +148,7 @@ def user_factory(username):
     result = None
     user = None
     if user_file.exists():
-        result = BaseUser.load(user_file)
+        result = BaseUser.load(user_file, **kwargs)
     if type(result) == tuple:
         user, card_file = result
         logger.debug(f"Loading card from {card_file}")
@@ -154,3 +156,56 @@ def user_factory(username):
     else:
         user = result
     return user
+
+
+def ipa_factory():
+    """
+    Create a new IPAServerCA object.
+
+    .. note: Creating new IPA server with CA is not supported.
+
+    :return: object of IPAServerCA
+    :rtype: SCAutolib.models.CA.IPAServerCA
+    """
+    json_file = LIB_DUMP_CAS.joinpath("ipa-server.json")
+    if not json_file.exists():
+        msg = "Dump file for ipa server CA is not present."
+        logger.error(msg)
+        logger.error("The reason for this is most likely that the system was "
+                     "not configured for IPA client via SCAutolib")
+        raise SCAutolibException(msg)
+    ca = BaseCA.load(json_file)
+    if not isinstance(ca, IPAServerCA):
+        msg = "Values in dump file are not valid for IPA server, so the " \
+              "object can't be created"
+        logger.error(msg)
+        raise SCAutolibException(msg)
+    return ca
+
+
+def local_ca_factory(path: Path = None, force: bool = False):
+    """
+    Create a new LocalCA object or load existing one if no path provided.
+
+    :param path: path to the CA directory
+    :type path: Path
+    :param force: force creation of new CA
+    :type force: bool
+    :return: object of LocalCA
+    :rtype: SCAutolib.models.CA.LocalCA
+    """
+    if not path:
+        return BaseCA.load(LIB_DUMP_CAS.joinpath("local-ca.json"))
+
+    path.mkdir(exist_ok=True, parents=True)
+    cnf = OpensslCnf(path.joinpath("ca.cnf"), "CA", str(path))
+    ca = LocalCA(root_dir=path, cnf=cnf)
+    if force:
+        logger.warning(f"Removing previous local CA in a directory {path}")
+        ca.cleanup()
+    ca.dump_file = path.joinpath("ca-dump.json")
+    cnf.create()
+    cnf.save()
+    ca.setup()
+    run(["systemctl", "restart", "sssd"], sleep=5)
+    return ca
